<?php

/**
 * This class contains all the fields used in the widget "Progress Map List & Filter" 
 *
 * @version 1.0 
 */
 
if(!defined('ABSPATH')){
    exit; // Exit if accessed directly
}

if( !class_exists( 'CspmlMetabox' ) ){
	
	class CspmlMetabox{
		
		private $plugin_path;
		private $plugin_url;
				
		private static $_this;	
		
		/**
		 * [@object_type] The name of the post to which we'll add the metaboxes
		 * @since 1.0 */
		 
		public $object_type;
		
		public $plugin_settings = array();
		
		protected $metafield_prefix;
		
		public $selected_cpt;
		
		function __construct($atts = array()){
			
			extract( wp_parse_args( $atts, array(
				'plugin_path' => '', 
				'plugin_url' => '',
				'object_type' => '',
				'plugin_settings' => array(), 
				'metafield_prefix' => '',
			)));
             
			self::$_this = $this;       
				           
			$this->plugin_path = $plugin_path;
			$this->plugin_url = $plugin_url;
			
			$this->plugin_settings = $plugin_settings;
			
			$this->metafield_prefix = $metafield_prefix;
			
			$this->object_type = $object_type;
			
			/**
			 * Get selected post type based on the post ID */
			 
			$post_id = 0;
			
			if(isset($_REQUEST['post'])){
				
				$post_id = $_REQUEST['post'];
			
			}elseif(isset($_REQUEST['post_ID'])){
				
				$post_id = $_REQUEST['post_ID'];
			}

			$this->selected_cpt = get_post_meta( $post_id, $this->metafield_prefix . '_post_type', true );

		}

		static function this() {
			
			return self::$_this;
		
		}

		
		/**
		 * "Progress Map List & Filter" Metabox.
		 * This metabox will contain all the settings needed for "Progress Map List & Filter"
		 *
		 * @since 1.0
		 */
		function cspml_list_and_filter_metabox(){
			
			$cspml_metabox_options = array(
				'id'            => $this->metafield_prefix . '_pmlf_metabox',
				'title'         => '<img src="'.$this->plugin_url.'/img/list-and-filter-icon.png" style="width:20px; margin:0 10px -3px 0;" />'.__( 'List & Filter Settings', 'cspml' ),
				'object_types'  => array( $this->object_type ), // Post type
				'show_on_cb'   => function(){
					global $pagenow;
					return ($pagenow == 'post-new.php') ? false : true;
				},
				'show_names' => true, // Show field names on the left
			);
			
			/**
			 * Metabox */
			 
			$cspml_metabox = new_cmb2_box( $cspml_metabox_options );

			/** 
			 * Check if we'll use the extension "List & Filter" or not */
			 
			$cspml_metabox->add_field( array(
				'id'   => $this->metafield_prefix . '_list_ext',
				'name' => 'Activate/Deactivate',
				'desc' => 'Check this option if you want to activate the extension "List & Filter". This will replace the carousel of "Progress Map" with a list and a filter.',
				'type' => 'checkbox',
				'before_row' => '<div class="cspml_single_field">',
				'after_row' => '</div>',
			) );

			/**
			 * Display the settings */
			 
			$this->cspml_list_and_filter_settings_tabs($cspml_metabox, $cspml_metabox_options);
			
		}
		
		
		/**
		 * Buill all the tabs that contains "Progress Map List & Filter" settings
		 *
		 * @since 1.0
		 */
		function cspml_list_and_filter_settings_tabs($metabox_object, $metabox_options){

			/**
			 * setting tabs */
			 
			$tabs_setting = array(
				'args' => $metabox_options,
				'tabs' => array()
			);
				
				/**
				 * Tabs array */
				 
				$cspml_tabs = array(
					
					/**
				 	 * Layout Settings */
					 
					array(
						'id' => 'list_layout_settings', 
						'title' => 'Layout settings', 
						'callback' => 'cspml_layout_fields'
					),

					/**
					 * Options bar Settings */
					 
					array(
						'id' => 'options_bar_settings',
						'title' => 'Options bar settings',
						'callback' => 'cspml_options_bar_fields',
					),
	
					/**
					 * List items Settings */
					 
					array(
						'id' => 'list_items_settings',
						'title' => 'List items settings',
						'callback' => 'cspml_list_items_fields',
					),
				
	
					/**
					 * Sort options Settings */
					 
					array(
						'id' => 'sort_options_settings',
						'title' => 'Sort options settings',
						'callback' => 'cspml_sort_options_fields',
					),
				
	
					/**
					 * Pagination Settings */
					 
					array(
						'id' => 'pagination_settings',
						'title' => 'Pagination settings',
						'callback' => 'cspml_pagination_fields',
					),
				
	
					/**
					 * Search filter Settings */
					 
					array(
						'id' => 'search_filter_settings',
						'title' => 'Search filter settings',
						'callback' => 'cspml_search_filter_fields',
					),
						
				);
				
				foreach($cspml_tabs as $tab_data){
				 
					$tabs_setting['tabs'][] = array(
						'id'     => 'cspml_' . $tab_data['id'],
						'title'  => '<span class="cspm_tabs_menu_image"><img src="'.$this->plugin_url.'/img/admin-icons/'.str_replace('_', '-', $tab_data['id']).'.png" style="width:20px;" /></span> <span class="cspm_tabs_menu_item">'.__( $tab_data['title'], 'cspm' ).'</span>',						
						'fields' => call_user_func(array(&$this, $tab_data['callback'])),
					);
		
				}
									
			/*
			 * set tabs */
			 
			$metabox_object->add_field( array(
				'id'   => 'cspml_pmlf_settings_tabs',
				'type' => 'tabs',
				'tabs' => $tabs_setting
			) );
			
			return $metabox_object;
			
		}
		
		
		/**
		 * Layout Fields
		 *
		 * @since 1.0 
		 */
		function cspml_layout_fields(){
			
			$fields = array();
			
			$fields[] = array(
				'name' => 'Layout Settings',
				'desc' => '',
				'type' => 'title',
				'id'   => $this->metafield_prefix . '_list_layout_settings',
				'attributes' => array(
					'style' => 'font-size:20px; color:#008fed; font-weight:400;'
				),
			);
			
			$fields[] = array(
				'id' => $this->metafield_prefix . '_list_layout',
				'name' => 'Layout Type',
				'desc' => 'Choose a layout type.',
				'type' => 'radio',
				'default' => 'vertical',
				'options' => array(
					'vertical' => 'Vertical',
					'horizontal-left' => 'Horizontal - Map on the Left, List & Filter on the Right',
					'horizontal-right' => 'Horizontal - Map on the Right, List & Filter on the Left',
				)
			);
			
			$fields[] = array(
				'id' => $this->metafield_prefix . '_map_height',
				'name' => 'Map Height',
				'desc' => 'Specify the height of the map in pixels. Defaults to "400"',
				'type' => 'text',
				'default' => '400',
				'attributes' => array(
					'type' => 'number',
					'pattern' => '\d*',
					'min' => '0',
				),								
			);
			
			$fields[] = array(
				'id' => $this->metafield_prefix . '_list_height',
				'name' => 'List Max-Height',
				'desc' => 'Specify the height of the list in pixels. By default, the list has no height!',
				'type' => 'text',
				'default' => '',
				'attributes' => array(
					'type' => 'number',
					'pattern' => '\d*',
					'min' => '0',
				),								
			);
				
			return $fields;
			
		}
		
		
		/**
		 * Options Bar Fields
		 *
		 * @since 1.0 
		 */
		function cspml_options_bar_fields(){
			
			$fields = array();
			
			$fields[] = array(
				'name' => 'Options Bar Settings',
				'desc' => '',
				'type' => 'title',
				'id'   => $this->metafield_prefix . '_options_bar_settings',
				'attributes' => array(
					'style' => 'font-size:20px; color:#008fed; font-weight:400;'
				),
			);
			
			$fields[] = array(
				'id' => $this->metafield_prefix . '_show_options_bar',
				'name' => __('Options Bar Visibility', 'cspml'),				
				'type' => 'radio',
				'desc' => __('Show/Hide the Options bar. The "Options bar" is the top area containing the "View options", the "Sort by" option and the "Posts count".', 'cspml'),
				'options' => array(
					'yes' => __( 'Show', 'cspml' ),
					'no' => __( 'Hide', 'cspml' ),
				),
				'default' => 'yes',
			);
			
			$fields[] = array(
				'id'   => $this->metafield_prefix . '_view_options_section',
				'name' => __('View options settings', 'cspml' ),
				'type' => 'title',
				'attributes' => array(
					'style' => 'font-size:15px; color:#ff6600; font-weight:600;'
				),				
			);

				$fields[] = array(
					'id' => $this->metafield_prefix . '_show_view_options',
					'name' => __('View Options Visibility', 'cspml'),					
					'type' => 'radio',
					'desc' => __('Show/Hide the "View options". The "View options" is the botton in the Options bar that allows switching from Grid view to List view.', 'cspml'),
					'options' => array(
						'yes' => __( 'Show', 'cspml' ),
						'no' => __( 'Hide', 'cspml' ),
					),
					'default' => 'yes',
				);
			
				$fields[] = array(
					'id' => $this->metafield_prefix . '_default_view_option',
					'name' => __('Initial view', 'cspml'),					
					'type' => 'radio',
					'desc' => __('Select the initial view. Defaults to "List".', 'cspml'),
					'options' => array(
						'list' => __( 'List', 'cspml' ),
						'grid' => __( 'Grid', 'cspml' ),
					),
					'default' => 'list',
				);
				
				$fields[] = array(
					'id' => $this->metafield_prefix . '_grid_cols',
					'name' => __('Grid column', 'cspml'),					
					'type' => 'radio',
					'desc' => __('Select the number of items to display in a single row for the "Grid view".', 'cspml'),
					'options' => array(
						'cols1' => __( 'One column', 'cspml' ),
						'cols2' => __( 'Two columns', 'cspml' ),
						'cols3' => __( 'Three columns', 'cspml' ),
						'cols4' => __( 'Four columns', 'cspml' ),
						'cols6' => __( 'Six columns', 'cspml' ),
					),
					'default' => 'cols3',
				);
				
			$fields[] = array(
				'id'   => $this->metafield_prefix . '_post_count_section',
				'name' => __('Posts count settings', 'cspml' ),
				'type' => 'title',
				'attributes' => array(
					'style' => 'font-size:15px; color:#ff6600; font-weight:600;'
				),			
			);

				$fields[] = array(
					'id' => $this->metafield_prefix . '_cslf_show_posts_count',				
					'name' => __('Posts Count Visibility', 'cspml'),
					'type' => 'radio',
					'desc' => __('Show/Hide the "Posts count".', 'cspml'),
					'options' => array(
						'yes' => __( 'Show', 'cspml' ),
						'no' => __( 'Hide', 'cspml' ),
					),
					'default' => 'yes',
				);
				
				$fields[] = array(
					'id'      => $this->metafield_prefix . '_cslf_posts_count_clause',
					'name'    => __('Posts count label', 'cspml'),
					'desc' => __('Use this field to enter your custom label.<br /><strong>Syntaxe:</strong> YOUR LABEL [posts_count] YOUR LABEL', 'cspml'),
					'default' => '[posts_count] Result(s)',					
					'type'    => 'text',
				);
				
			return $fields;
			
		}
		
		
		/**
		 * List Items Fields
		 *
		 * @since 1.0 
		 */
		function cspml_list_items_fields(){
			
			$fields = array();
			
			$fields[] = array(
				'name' => 'List Items Settings',
				'desc' => '',
				'type' => 'title',
				'id'   => $this->metafield_prefix . '_list_items_settings',
				'attributes' => array(
					'style' => 'font-size:20px; color:#008fed; font-weight:400;'
				),
			);
			
			$fields[] = array(
				'id' => $this->metafield_prefix . '_scroll_to_list_item',
				'name' => 'Scroll to list item',
				'desc' => 'Scroll to a list item when its related marker on the map is clicked. Defaults to "Yes".',
				'type' => 'radio',
				'default' => 'yes',
				'options' => array(
					'yes' => 'Yes',
					'no' => 'No',
				)
			);
			
			$fields[] = array(
				'id' => $this->metafield_prefix . '_listings_title',
				'name' => __('List Item Title', 'cspml'),
				'desc' => __('Create your customized title by entering the name of your custom syntaxe. You can combine the title with your custom fields. Leave this field empty to use the default title.
					<br /><strong>Syntax:</strong> [meta_key<sup>1</sup>][separator<sup>1</sup>][meta_key<sup>2</sup>][separator<sup>2</sup>][meta_key<sup>n</sup>]...[title lenght].
					<br /><strong>Example of use:</strong> [property_price][s=,][-][title][l=50]
					<br /><strong>*</strong> To add the title enter [title]
					<br /><strong>*</strong> To insert empty an space enter [-]
					<br /><strong>* Make sure there\'s no empty spaces between ][</strong>', 'cspml'),				
				'type' => 'textarea'
			);

			$fields[] = array(
				'id' => $this->metafield_prefix . '_listings_details',
				'name' => __('List Item Description', 'cspml'),
				'desc' => __('Create your customized description content. You can combine the content with your custom fields & taxonomies. Leave this field empty to use the default description.
					<br /><strong>Syntax:</strong> [content;content_length][separator][t=label:][meta_key][separator][t=Category:][tax=taxonomy_slug][separator]...[description length]
					<br /><strong>Example of use:</strong> [content;80][s=br][t=Category:][-][tax=category][s=br][t=Address:][-][post_address]
					<br /><strong>*</strong> To specify a description length, use <strong>[l=LENGTH]</strong>. Change LENGTH to a number (e.g. 100).
					<br /><strong>*</strong> To add a label, use <strong>[t=YOUR_LABEL]</strong>
					<br /><strong>*</strong> To add a custom field, use <strong>[CUSTOM_FIELD_NAME]	</strong>				
					<br /><strong>*</strong> To insert a taxonomy, use <strong>[tax=TAXONOMY_SLUG]</strong>
					<br /><strong>*</strong> To insert new line enter <strong>[s=br]</strong>
					<br /><strong>*</strong> To insert an empty space enter <strong>[-]</strong>
					<br /><strong>*</strong> To insert the content/excerpt, use <strong>[content;LENGTH]</strong>. Change LENGTH to a number (e.g. 100).
					<br /><strong>* Make sure there\'s no empty spaces between ][</strong>', 'cspml'),
				'default' => '[l=700]',
				'type' => 'textarea'
			);
			
			$fields[] = array(
				'id' => $this->metafield_prefix . '_cslf_click_on_title',
				'name' => 'Title as link',
				'desc' => 'Select "Yes" to use the title as a link to the post page.',
				'type' => 'radio',
				'default' => 'yes',
				'options' => array(
					'yes' => 'Yes',
					'no' => 'No',
				)
			);
			
			$fields[] = array(
				'id' => $this->metafield_prefix . '_cslf_click_on_img',
				'name' => 'Image as link',
				'desc' => 'Select "Yes" to use the featured image as a link to the post page.',
				'type' => 'radio',
				'default' => 'yes',
				'options' => array(
					'yes' => 'Yes',
					'no' => 'No',
				)
			);
			
			$fields[] = array(
				'id' => $this->metafield_prefix . '_cslf_external_link',
				'name' => 'Post URL',
				'desc' => 'Choose the way you want to open the post page.',
				'type' => 'radio',
				'default' => 'same_window',
				'options' => array(
					'new_window' => 'Open in a new window',
					'same_window' => 'Open in the same window'
				)
			);
			
			$fields[] = array(
				'id' => $this->metafield_prefix . '_marker_position_btn_section',
				'name' => '"Position on the map" Button',
				'desc' => '',
				'type' => 'title',
				'attributes' => array(
					'style' => 'font-size:15px; color:#ff6600; font-weight:600;'
				),
			);
			
			$fields[] = array(
				'id' => $this->metafield_prefix . '_show_fire_pinpoint_btn',
				'name' => 'Button Visibilty',
				'desc' => 'Show/Hide the button that shows the item\'s location on the map.',
				'type' => 'radio',
				'default' => 'yes',
				'options' => array(
					'yes' => 'Show',
					'no' => 'Hide',
				)
			);
		
			return $fields;
					
		}
		
		
		/**
		 * Sort Options settings Fields
		 *
		 * @since 1.0 
		 */
		function cspml_sort_options_fields(){
			
			$fields = array();
			
			$fields[] = array(
				'name' => 'Sort Options Settings',
				'desc' => '',
				'type' => 'title',
				'id'   => $this->metafield_prefix . '_sort_options_settings',
				'attributes' => array(
					'style' => 'font-size:20px; color:#008fed; font-weight:400;'
				),
			);
			
			$fields[] = array(
				'id' => $this->metafield_prefix . '_show_sort_option',
				'name' => __('Turn On/Off the sort option', 'cspml'),				
				'type' => 'radio',
				'desc' => __('Enable/Disable the sort option.', 'cspml'),
				'options' => array(
					'yes' => __( 'Enable', 'cspml' ),
					'no' => __( 'Disable', 'cspml' ),
				),
				'default' => 'yes',
			);

			$fields[] = array(
				'id' => $this->metafield_prefix . '_sort_options',
				'name' => __('Default Sort Options', 'cspml'),				
				'type' => 'pw_multiselect',
				'desc' => __('Select the sort options to use from the available options.', 'cspml'),
				'options' => array(
					'default|Default|init' => __( 'Default', 'cspml' ),
					'data-date-created|Date (Newest first)|desc' => __( 'Date (Newest first)', 'cspml' ),
					'data-date-created|Date (Oldest first)|asc' => __( 'Date (Oldest first)', 'cspml' ),
					'data-title|Title A to Z|asc' => __( 'Title A to Z', 'cspml' ),
					'data-title|Title Z to A|desc' => __( 'Title Z to A', 'cspml' )
				),
				'default' => array(),
				'attributes' => array(
					'placeholder' => 'Select'
				),
			);
			
			$fields[] = array(
				'id' => $this->metafield_prefix . '_sort_options_order',
				'name' => __( 'Sort options display order', 'cspml' ),
				'desc' => __( 'Change the display order of the sort options.', 'cspml' ),				
				'type' => 'order',
				//'inline' => true,
				'options' => array(
					'default' => __('Default sort option', 'cspml'),
					'custom' => __('Custom sort options', 'cspml'),					
				),
			);
						
			$fields[] = array(
				'id' => $this->metafield_prefix . '_custom_sort_options',
				'name' => 'Custom Sort Options', 
				'desc' => 'Add your custom sort options. You can sort list items by custom fields.<br />
						   <strong>!Important:</strong> The data are not going to be saved automatically in the Database, you still have to click the "Save" button below to 
						   confirm your data. You can add as many custom sort options before saving.',
				'type' => 'group',
				'repeatable'  => true,
				'options'     => array(
					'group_title'   => __( 'Custom field {#}', 'cspml' ),
					'add_button'    => __( 'Add New Custom Field', 'cspml' ),
					'remove_button' => __( 'Remove Custom Field', 'cspml' ),
					'sortable'      => true,
					'closed'     => true,
				),
				'fields' => array(	
					array(
						'id' => 'sort_options_name',
						'name' => 'Custom field name', 
						'desc' => 'The name of the custom field to sort with. e.g. "property_price"',
						'type' => 'text',
						'default' => '',
					),	
					array(
						'id' => 'sort_meta_type',
						'name' => 'Custom field type', 
						'desc' => 'Select the custom field type.',
						'type' => 'select',
						'default' => 'CHAR',
						'options' => array(
							'CHAR' => 'CHAR',
							'NUMERIC' => 'NUMERIC',
							'BINARY' => 'BINARY',							
							'DATE' => 'DATE',
							'DATETIME' => 'DATETIME',
							'DECIMAL' => 'DECIMAL',
							'SIGNED' => 'SIGNED',
							'TIME' => 'TIME',
							'UNSIGNED' => 'UNSIGNED',
						)
					),
					array(
						'id' => 'sort_options_label',
						'name' => 'Label', 
						'desc' => 'The label to use to describe the sort option. e.g. "Price"',
						'type' => 'text',
						'default' => '',	
						'attributes'  => array(
							'data-group-title' => 'text'
						)																
					),			
					array(
						'id' => 'sort_options_order',
						'name' => 'Sort order', 
						'desc' => 'Select the sort order.',
						'type' => 'multicheck',
						'default' => array('desc', 'asc'),
						'options' => array(
							'desc' => 'Descending <sup>e.g. (Highest first)</sup>',
							'asc' => 'Ascending <sup>e.g. (Lowest first)</sup>'
						),
						'select_all_button' => false,
					),
					array(
						'id' => 'sort_options_desc_suffix',
						'name' => 'Descending suffix', 
						'desc' => 'Enter the suffix to add to the end of the label for the descending order. e.g. "(Highest first)"',
						'type' => 'text',
						'default' => '(Highest first)',
					),
					array(
						'id' => 'sort_options_asc_suffix',
						'name' => 'Ascending suffix', 
						'desc' => 'Enter the suffix to add to the end of the label for the ascending order. e.g. "(Lowest first)"',
						'type' => 'text',
						'default' => '(Lowest first)',
					),
					array(
						'id' => 'sort_options_visibilty',
						'name' => 'Visibilty', 
						'desc' => 'Show/hide the custom sort option from being displayed. Default "Show"',
						'type' => 'radio',
						'default' => 'yes',
						'options' => array(
							'yes' => 'Show',
							'no' => 'Hide'
						)
					)
				)
			);
		
			return $fields;
			
		}
		
		
		/**
		 * Pagination settings Fields
		 *
		 * @since 1.0 
		 */
		function cspml_pagination_fields(){
			
			$fields = array();
			
			$fields[] = array(
				'name' => 'Pagination Settings',
				'desc' => '',
				'type' => 'title',
				'id'   => $this->metafield_prefix . '_pagination_settings',
				'attributes' => array(
					'style' => 'font-size:20px; color:#008fed; font-weight:400;'
				),
			);
			
			$fields[] = array(
				'id'   => $this->metafield_prefix . '_posts_per_page',
				'name' => __( 'Number of posts per page', 'cspml' ),				
				'desc' => __( 'Enter the number of the posts per page. Defaults to the number selected in "Settings / Reading / Blog pages show at most".', 'cspml' ),
				'type' => 'text',
				'attributes' => array(
					'type' => 'number',
					'pattern' => '\d*',
					'min' => '0',
				),				
			);

			$fields[] = array(
				'id' => $this->metafield_prefix . '_pagination_position',
				'name' => __('Pagination position', 'cspml'),				
				'type' => 'radio',
				'desc' => __('Select the position of the pagination in the page. Defaults to "Bottom".', 'cspml'),
				'options' => array(
					'top' => __( 'Top', 'cspml' ),
					'bottom' => __( 'Bottom', 'cspml' ),
					'both' => __( 'Both', 'cspml' ),
				),
				'default' => 'bottom',
			);

			$fields[] = array(
				'id' => $this->metafield_prefix . '_pagination_align',
				'name' => __('Pagination alignment', 'cspml'),				
				'type' => 'radio',
				'desc' => __('Select the alignment of the pagination in the page. Defaults to "Center".', 'cspml'),
				'options' => array(
					'left' => __( 'Left', 'cspml' ),
					'center' => __( 'Center', 'cspml' ),
					'right' => __( 'Right', 'cspml' ),
				),
				'default' => 'center',
			);

			$fields[] = array(
				'id'   => $this->metafield_prefix . '_prev_page_text',
				'name' => __( 'Previous page text', 'cspml' ),				
				'desc' => __( 'Enter the text that appears in the pagination button "Previous". Defaults to "&laquo;".', 'cspml' ),
				'type' => 'text',
				'default' => '&laquo;',
			);

			$fields[] = array(
				'id' => $this->metafield_prefix . '_next_page_text',
				'name' => __( 'Next page text', 'cspml' ),				
				'desc' => __( 'Enter the text that appears in the pagination button "Next". Defaults to "&raquo;".', 'cspml' ),
				'type' => 'text',
				'default' => '&raquo;',
			);

			$fields[] = array(
				'id' => $this->metafield_prefix . '_show_all',
				'name' => __('Show all pages', 'cspml'),				
				'type' => 'radio',
				'desc' => __('Select True to show all of the pages in the pagination instead of a short list of pages near the current page. Defaults to "False".', 'cspml'),
				'options' => array(
					'true' => __( 'Yes', 'cspml' ),
					'false' => __( 'No', 'cspml' ),
				),
				'default' => 'false',
			);

			return $fields;
			
		}
		
		
		/**
		 * Search Filter settings Fields
		 *
		 * @since 1.0 
		 */
		function cspml_search_filter_fields(){
			
			$fields = array();
			
			$fields[] = array(
				'name' => 'Search Filter Settings',
				'desc' => '',
				'type' => 'title',
				'id'   => $this->metafield_prefix . '_search_filter_settings',
				'attributes' => array(
					'style' => 'font-size:20px; color:#008fed; font-weight:400;'
				),
			);
						
			$fields[] = array(
				'id' => $this->metafield_prefix . '_cslf_faceted_search_option',
				'name' => __('Search Filter option', 'cspml'),				
				'type' => 'radio',
				'desc' => __('Select "Yes" to enable this option in the plugin. Defaults to "No".', 'cspml'),
				'options' => array(
					'yes' => __( 'Yes', 'cspml' ),
					'no' => __( 'No', 'cspml' ),
				),
				'default' => 'yes',
			);

			$fields[] = array(
				'id' => $this->metafield_prefix . '_faceted_search_position',
				'name' => __('Search Filter position', 'cspml'),				
				'type' => 'radio',
				'desc' => __('Choose the position of the search filter on a page. Defaults to "Left".', 'cspml'),
				'options' => array(
					'left' => __( 'Left', 'cspml' ),
					'right' => __( 'Right', 'cspml' ),
				),
				'default' => 'left',
			);

			$fields[] = array(
				'id' => $this->metafield_prefix . '_faceted_search_display_option',
				'name' => __('Search Filter display option', 'cspml'),				
				'type' => 'radio',
				'desc' => __('Choose whether you want to hide/show the filter on page load. Defaults to "Show on page load".', 'cspml'),
				'options' => array(
					'show' => __( 'Show on page load', 'cspml' ),
					'hide' => __( 'Hide on page load', 'cspml' ),
				),
				'default' => 'show',
			);
			
			$fields[] = array(
				'id' => $this->metafield_prefix . '_filter_btns_position',
				'name' => __('Filter & Reset buttons position', 'cspml'), 
				'desc' => 'Select the position of the filter buttons in the page. Defaults to "Bottom".',
				'type' => 'radio',
				'default' => 'bottom',
				'options' => array(
					'top' => 'Top',
					'bottom' => 'Bottom',
					'both' => 'Both',
				)
			);	
			
			$fields[] = array(
				'id' => $this->metafield_prefix . '_filter_fields_order',
				'name' => __( 'Filter fields display order', 'cspml' ),
				'desc' => __( 'Change the display order of the filter fields.', 'cspml' ),				
				'type' => 'order',
				//'inline' => true,
				'options' => array(
					'keyword' => __('keyword', 'cspml'),
					'date_filter' => __('Posts publish date', 'cspml'),
					'taxonomies' => __('Taxonomies', 'cspml'),
					'custom_fields' => __('Custom Fields', 'cspml'),
				),
			);
			
			/**
			 * Keyword search
			 * @since 2.2 */
			 	
			$fields[] = array(
				'id' => $this->metafield_prefix . '_cslf_keyword_param_section',
				'name' => 'Keyword filter Parameters',
				'desc' => 'Filter posts based on a keyword.',
				'type' => 'title',
				'attributes' => array(
					'style' => 'font-size:15px; color:#ff6600; font-weight:600;'
				),
			);
			
			$fields[] = array(
				'id' => $this->metafield_prefix . '_keyword_filter_option',
				'name' => 'Keyword filter', 
				'desc' => 'This will add a text field in the filter that allows to filter posts based on a keyword. Defaults to "No".',
				'type' => 'radio',
				'default' => 'no',
				'options' => array(
					'yes' => 'Yes',
					'no' => 'No',
				)
			);
			
			$fields[] = array(
				'id' => $this->metafield_prefix . '_keyword_display_status',
				'name' => 'Display status',
				'desc' => 'Choose whether to open this field container on page load or to close it. Defaults to "Open".',
				'type' => 'radio',
				'default' => 'open',
				'options' => array(
					'open' => 'Open',
					'close' => 'Close'
				)
			);
						
			$fields[] = array(
				'id' => $this->metafield_prefix . '_keyword_filter_label',
				'name' => 'Title', 
				'desc' => 'Enter a Title to display in the filter. Defaults to "Keyword".',
				'type' => 'text',
				'default' => 'Keyword'												
			);
			
			$fields[] = array(
				'id' => $this->metafield_prefix . '_keyword_filter_placeholder',
				'name' => 'Text field placeholder', 
				'desc' => 'Enter a text to display as the placehoder of the text field. Defaults to "Keyword".',
				'type' => 'text',
				'default' => 'Keyword'												
			);
			
			$fields[] = array(
				'id' => $this->metafield_prefix . '_keyword_filter_description',
				'name' => 'Description', 
				'desc' => 'The description will be displayed above the field.',
				'type' => 'textarea',
				'default' => '',
				'attributes' => array(
					'rows' => 3,
				)											
			);
			
			/**
			 * Date Parameters 
			 * @since 2.1 */
			 	
			$fields[] = array(
				'id' => $this->metafield_prefix . '_cslf_date_param_section',
				'name' => 'Posts Publish Date Parameters',
				'desc' => 'Filter posts by date period.',
				'type' => 'title',
				'attributes' => array(
					'style' => 'font-size:15px; color:#ff6600; font-weight:600;'
				),
			);
			
			$fields[] = array(
				'id' => $this->metafield_prefix . '_date_filter_option',
				'name' => 'Filter posts by publish date', 
				'desc' => 'This will add a Datepicker field in the filter that allows to filter posts based on the publish date. Defaults to "No".',
				'type' => 'radio',
				'default' => 'no',
				'options' => array(
					'yes' => 'Yes',
					'no' => 'No',
				)
			);
			
			$fields[] = array(
				'id' => $this->metafield_prefix . '_date_filter_display_status',
				'name' => 'Display status',
				'desc' => 'Choose whether to open this field container on page load or to close it. Defaults to "Open".',
				'type' => 'radio',
				'default' => 'open',
				'options' => array(
					'open' => 'Open',
					'close' => 'Close'
				)
			);
						
			$fields[] = array(
				'id' => $this->metafield_prefix . '_date_filter_param',
				'name' => 'Posts publish date parameters', 
				'desc' => 'Choose the parameters of your datepicker field',
				'type' => 'group',
				'default' => '',
				'repeatable' => false,
				'options'     => array(
					'group_title'   => __( 'Datepicker parameters', 'cspml' ),
					'closed'     => true,
				),				
				'fields' => array(
					array(
						'id' => 'date_filter_label',
						'name' => 'Title', 
						'desc' => 'Enter a Title to display in the filter. Defaults to "Posts publish date".',
						'type' => 'text',
						'default' => 'Posts publish date'												
					),
					array(
						'id' => 'date_filter_description',
						'name' => 'Description', 
						'desc' => 'The description will be displayed above the field.',
						'type' => 'textarea',
						'default' => '',
						'attributes' => array(
							'rows' => 3,
						)											
					),
					array(
						'id' => 'date_filter_type',
						'name' => 'Filter by', 
						'desc' => 'Choose how you want to get posts. Defaults to "Exact date".',
						'type' => 'radio',
						'default' => 'exact_date',
						'options' => array(
							'exact_date' => 'Exact date <sup>(Show posts published in the selected date)</sup>',							
							'after_date' => 'After date <sup>(Show posts published after selected date)<sup>',
							'before_date' => 'Before date <sup>(Show posts published before selected date)<sup>',
							'date_range' => 'Between two dates <sup>(Show posts published between two dates)</sup>',
						)
					),
					array(
						'id' => 'date_format',
						'name' => 'Date format', 
						'desc' => 'Choose the date format. Defaults to "Year, Month, Day".',
						'type' => 'radio',
						'default' => 'ymd',
						'options' => array(
							'ymd' => 'Year, Month, Day',
							'dmy' => 'Day, Month, Year',
						)
					),
					array(
						'id' => 'exclude_date_atts',
						'name' => 'Exclude date attributes', 
						'desc' => 'You can exclude up to two attributes from the date ("Day", "Month" and/or "Year").',
						'type' => 'pw_multiselect',
						'default' => '',
						'options' => array(
							'y' => 'Year',
							'm' => 'Month',
							'd' => 'Day',
						),
						'attributes' => array(
							'placeholder' => 'Select date attribute',
							'data-maximum-selection-length' => '2',
						),
					),
					array(
						'id' => 'start_date',
						'name'   => 'Start date (Year/Month/Day)',
						'desc' => 'The start view date. All the dates before this date will be disabled.',
						'type' => 'text_date',
						'date_format' => 'Y/m/d',
					),
					array(
						'id' => 'end_date',
						'name'   => 'End date (Year/Month/Day)',
						'desc' => 'The end view date. All the dates after this date will be disabled.',
						'type' => 'text_date',
						'date_format' => 'Y/m/d',
					), 
					array(
						'id' => 'start_view',
						'name' => 'Start View', 
						'desc' => 'Select the start view when the datepicker is loaded. Defaults to "Days".',
						'type' => 'radio',
						'default' => 'days',
						'options' => array(
							'days' => 'Days',
							'months' => 'Months',
							'years' => 'Years',
						)
					), 
					array(
						'id' => 'week_start',
						'name' => 'Week start',
						'desc' => 'Select the start day of the week. Defaults to "Days".',
						'type' => 'radio',
						'default' => 'sunday',
						'options' => array(
							'sunday' => 'Sunday',
							'monday' => 'Monday',
							'tuesday' => 'Tuesday',
							'wednesday' => 'Wednesday',
							'thursday' => 'Thursday',
							'friday' => 'Friday',
							'saturday' => 'Saturday',
						)
					),
					array(
						'id' => 'year_first',
						'name' => 'Year first', 
						'desc' => 'Show year before month on the datepicker header. Defaults to "No".',
						'type' => 'radio',
						'default' => 'no',
						'options' => array(							
							'yes' => 'Yes',
							'no' => 'No',
						)
					),
					array(
						'id' => 'year_suffix',
						'name' => 'Year suffix', 
						'desc' => 'Add suffix to year on the datepicker header.',
						'type' => 'text',
						'default' => '',
					),
					array(
						'id' => 'language',
						'name' => 'Language',
						'desc' => 'Select datepicker language. Defaults to "English".',
						'type' => 'select',
						'default' => 'en-US',
						'options' => array(
							'ca-ES' => 'Catalan', 
							'zh-CN' => 'Chinese - China',
							'nl-NL' => 'Dutch - The Netherlands',
							'en-US' => 'English - United States',
							'fr-FR' => 'French - France',
							'de-DE' => 'German - Germany', 
							'pt-BR' => 'Portuguese - Brazil',  
							'es-ES' => 'Spanish - Spain', 
							'sv-SE' => 'Swedish - Sweden', 
							'tr-TR' => 'Turkish - Turkey', 
						)
					),
					array(
						'id' => 'auto_show',
						'name' => 'Auto show', 
						'desc' => 'Show the datepicker automatically when initialized. Defaults to "No".',
						'type' => 'radio',
						'default' => 'no',
						'options' => array(							
							'yes' => 'Yes',
							'no' => 'No',
						)
					),
					array(
						'id' => 'auto_hide',
						'name' => 'Auto hide', 
						'desc' => 'Hide the datepicker automatically when picked. Defaults to "No".',
						'type' => 'radio',
						'default' => 'no',
						'options' => array(							
							'yes' => 'Yes',
							'no' => 'No',
						)
					),
					array(
						'id' => 'inline',
						'name' => 'Inline', 
						'desc' => 'Enable inline mode. When set to "Yes", datepicker will be appended in the filter container. Defaults to "No".',
						'type' => 'radio',
						'default' => 'no',
						'options' => array(							
							'yes' => 'Yes',
							'no' => 'No',
						)
					),
				)
			);
			
			/**
			 * Taxonomies */
			 			
			$fields[] = array(
				'id' => $this->metafield_prefix . '_cslf_taxonomies_section',
				'name' => 'Taxonomies Parameters',
				'desc' => '',
				'type' => 'title',
				'attributes' => array(
					'style' => 'font-size:15px; color:#ff6600; font-weight:600;'
				),
			);
			
			/**
			 * [@post_type_taxonomy_options] : Takes the list of all taxonomies related to the post type selected in "Query settings" */
			
			$post_type_taxonomy_options	= $this->cspml_get_post_type_taxonomies($this->selected_cpt);		
				unset($post_type_taxonomy_options['post_format']);
				
			reset($post_type_taxonomy_options); // Set the cursor to 0

			$taxonomy_fields = array();
			
			$group_id = $this->metafield_prefix . '_cslf_taxonomies';
			
			foreach($post_type_taxonomy_options as $cpt_taxonomy_slug => $cpt_taxonomy_title){
	
				$tax_name = $cpt_taxonomy_slug;
				$tax_label = $cpt_taxonomy_title;
				
				$taxonomy_fields[] = array(
					'id' => 'taxonomy_exclude_terms_'.$tax_name,
					'name' => 'Exclude terms',
					'desc' => 'Select the terms that you want to exclude from the filter.',
					'type' => 'pw_multiselect',
					'options' => $this->cspml_get_term_options($tax_name),
					'attributes' => array(
						'placeholder' => 'Select Term(s)',
						'data-conditional-id' => wp_json_encode( array( $group_id, 'taxonomy_name' ) ),
						'data-conditional-value' => $tax_name,																			
					),					
				);
				
				$taxonomy_fields[] = array(
					'id' => 'taxonomy_include_terms_'.$tax_name,
					'name' => 'Include terms',
					'desc' => 'Select the terms that you want to include in the filter. Defaults to all terms.',
					'type' => 'pw_multiselect',
					'options' => $this->cspml_get_term_options($tax_name),
					'attributes' => array(
						'placeholder' => 'Select Term(s)',
						'data-conditional-id' => wp_json_encode( array( $group_id, 'taxonomy_name' ) ),
						'data-conditional-value' => $tax_name,																									
					),
				);
				
			}				

			$fields[] = array(
				'id' => $group_id,
				'name' => 'Taxonomies', 
				'desc' => 'Add the taxonomies to use in the faceted search form.<br />
						   <strong>!Important:</strong> The data are not going to be saved automatically in the Database, you still have to click the "Save" button below to 
						   confirm your data. You can add as many taxonomies before saving.',
				'type' => 'group',
				'default' => '',
				'repeatable'  => true,
				'options'     => array(
					'group_title'   => __( 'Taxonomy {#}', 'cspml' ),
					'add_button'    => __( 'Add New Taxonomy', 'cspml' ),
					'remove_button' => __( 'Remove Taxonomy', 'cspml' ),
					'sortable'      => true,
					'closed'     => true,
				),				
				'fields' => array_merge(
					array(	
						array(
							'id' => 'taxonomy_label',
							'name' => 'Label/Title', 
							'desc' => 'Enter a label/Title to describe the taxonomy. e.g. "Category".',
							'type' => 'text',
							'default' => '',
							'attributes'  => array(
								'data-group-title' => 'text'
							)						
						),
						array(
							'id' => 'taxonomy_name',
							'name' => 'Taxonomy name', 
							'desc' => 'Enter the taxonomy name. e.g "category".',
							'type' => 'select',
							'options' => $post_type_taxonomy_options,
							'default' => '',					
						),	
						array(
							'id' => 'taxonomy_description',
							'name' => 'Description', 
							'desc' => 'The description will be displayed above the field.',
							'type' => 'textarea',
							'default' => '',
							'attributes' => array(
								'rows' => 3,
							)											
						)
					),
					
					$taxonomy_fields,
					
					array(
						array(
							'id' => 'taxonomy_operator_param',
							'name' => '"Operator" parameter', 
							'desc' => 'Operator to test. Possible values are "IN", "NOT IN", "AND".<a href="http://codex.wordpress.org/Class_Reference/WP_Query#Taxonomy_Parameters" target="_blank">More</a>',
							'type' => 'radio',
							'default' => 'IN',
							'options' => array(
								'AND' => 'AND',
								'IN' => 'IN',
								'NOT IN' => 'NOT IN',
							)
						),
						array(
							'id' => 'taxonomy_orderby_param',
							'name' => 'Orderby parameter', 
							'desc' => 'Order terms by. Defaults to "name".',
							'type' => 'select',
							'default' => 'name',
							'options' => array(
								'name' => 'Order by name',
								'term_group' => 'Order by term group',
								'term_id' => 'Order by term ID',
								'description' => 'Order by term description',
								'count' => 'Order by term count',
								'include' => 'Order by "Include term IDs"',
								'none' => 'No order',
							)
						),							
						array(
							'id' => 'taxonomy_order_param',
							'name' => 'Order parameter', 
							'desc' => 'Whether to order terms in ascending or descending order Defaults to "ASC".',
							'type' => 'radio',
							'default' => 'ASC',
							'options' => array(
								'ASC' => 'Ascending order from lowest to highest values',
								'DESC' => 'Descending order from highest to lowest values'
							)
						),							
						array(
							'id' => 'taxonomy_hide_empty',
							'name' => 'Hide empty terms', 
							'desc' => 'Hide the terms with no posts assigned to them. Default "Yes"',
							'type' => 'radio',
							'default' => 'yes',
							'options' => array(
								'yes' => 'Yes',
								'no' => 'No'
							)
						),
						array(
							'id' => 'taxonomy_search_all_option',
							'name' => '"Search All" Option', 
							'desc' => 'Select "Yes" to enable this option for this taxonomy. Default "No"',
							'type' => 'radio',
							'default' => 'no',
							'options' => array(
								'yes' => 'Yes',
								'no' => 'No'
							)
						),
						array(
							'id' => 'taxonomy_search_all_text',
							'name' => '"Search All" Label', 
							'desc' => 'Enter the label for the "Search all" option.',
							'type' => 'text',
							'default' => 'All',
						),
						array(
							'id' => 'taxonomy_symbol',
							'name' => 'Symbol', 
							'desc' => 'The symbol is the indicator of the taxonomy, like currency symbol ($) or surface unit (m²).',
							'type' => 'text',
							'default' => '',
						),
						array(
							'id' => 'taxonomy_symbol_position',
							'name' => 'Symbol position', 
							'desc' => 'Select in which side of the taxonomy the symbol will be displayed. Default (Before).',
							'type' => 'radio',
							'default' => 'before',
							'options' => array(
								'before' => 'Before',
								'after' => 'After'
							)
						),				
						array(
							'id' => 'taxonomy_show_count',
							'name' => 'Show count', 
							'desc' => 'Select "Yes" to show the number of posts assigned to each taxonomy term. Default "No".',
							'type' => 'radio',
							'default' => 'no',
							'options' => array(
								'yes' => 'Yes',
								'no' => 'No'
							)
						),
						array(
							'id' => 'taxonomy_display_type',
							'name' => 'Display type', 
							'desc' => 'Select the display type for this taxonomy. Default "checkbox".',
							'type' => 'radio',
							'default' => 'checkbox',
							'options' => array(
								'checkbox' => 'Checkbox',
								'radio' => 'Radio',
								'select' => 'Select',
								'number' => 'Number <sup>Use it only for numeric values</sup>',
								'double_slider' => 'Double range slider <sup>Use it only for numeric values</sup>',
								'single_slider' => 'Single range slider <sup>Use it only for numeric values</sup>',
							)
						),
						array(
							'id' => 'taxonomy_display_status',
							'name' => 'Display status',
							'desc' => 'Choose whether to open this field container on page load or to close it. Defaults to "Open".',
							'type' => 'radio',
							'default' => 'open',
							'options' => array(
								'open' => 'Open',
								'close' => 'Close'
							)
						),
						array(
							'id' => 'taxonomy_visibilty',
							'name' => 'Visibilty', 
							'desc' => 'Show/hide the taxonomy from being displayed in the filter form. Default "Show".',
							'type' => 'radio',
							'default' => 'yes',
							'options' => array(
								'yes' => 'Show',
								'no' => 'Hide'
							)
						)
					)
				)
			);
				
			$fields[] = array(
				'id' => $this->metafield_prefix . '_cslf_taxonomy_relation_param',
				'name' => '"Relation" parameter', 
				'desc' => 'Select the relationship between multiple taxonomies. Defaults to "AND". <a href="http://codex.wordpress.org/Class_Reference/WP_Query#Taxonomy_Parameters" target="_blank">More</a>',
				'type' => 'radio',
				'default' => 'AND',
				'options' => array(
					'AND' => 'AND',
					'OR' => 'OR',
				)
			);
			
			/**
			 * Custom fields */
			 	
			$fields[] = array(
				'id' => $this->metafield_prefix . '_cslf_custom_fields_section',
				'name' => 'Custom Fields Parameters',
				'desc' => '',
				'type' => 'title',
				'attributes' => array(
					'style' => 'font-size:15px; color:#ff6600; font-weight:600;'
				),
			);
			
			$group_id = $this->metafield_prefix . '_cslf_custom_fields';
			
			$fields[] = array(
				'id' => $group_id,
				'name' => 'Custom fields', 
				'desc' => 'Add the custom fields to use in the faceted search form.<br />
						   <strong>!Important:</strong> The data are not going to be saved automatically in the Database, you still have to click the "Save" button below to 
						   confirm your data. You can add as many custom fields before saving.',
				'type' => 'group',
				'default' => '',
				'repeatable' => true,
				'options'     => array(
					'group_title'   => __( 'Custom field {#}', 'cspml' ),
					'add_button'    => __( 'Add New Custom Field', 'cspml' ),
					'remove_button' => __( 'Remove Custom Field', 'cspml' ),
					'sortable'      => true,
					'closed'     => true,
				),				
				'fields' => array(
					array(
						'id' => 'custom_field_label',
						'name' => 'Label/Title', 
						'desc' => 'Enter a label/Title to describe the custom field. e.g. "Price".',
						'type' => 'text',
						'default' => '',
						'attributes'  => array(
							'data-group-title' => 'text'
						)							
					),
					array(
						'id' => 'custom_field_name',
						'name' => 'Custom field name',
						'desc' => 'Enter the name of custom field. e.g. "property_price"',
						'type' => 'text',
						'default' => '',
					),
					array(
						'id' => 'custom_field_description',
						'name' => 'Description', 
						'desc' => 'The description will be displayed above the field.',
						'type' => 'textarea',
						'default' => '',
						'attributes' => array(
							'rows' => 3,
						)											
					),					
					array(
						'id' => 'custom_field_symbol',
						'name' => 'Symbol', 
						'desc' => 'The symbol is the indicator of the custom field, like currency symbol ($) or surface unit (m²).',
						'type' => 'text',
						'default' => '',
					),
					array(
						'id' => 'custom_field_symbol_position',
						'name' => 'Symbol position', 
						'desc' => 'Select in which side of the custom field the symbol will be displayed. Default (Before).',
						'type' => 'radio',
						'default' => 'before',
						'options' => array(
							'before' => 'Before',
							'after' => 'After'
						)
					),
					array(
						'id' => 'custom_field_compare_param',
						'name' => '"Compare" parameter', 
						'desc' => 'Operator to test. Use it carefully. If you choose "BETWEEN", then your <u>"Display type"</u> should be <u>"Double range slider"</u>. More about compare, please visit. <a href="http://codex.wordpress.org/Class_Reference/WP_Query#Custom_Field_Parameters" target="_blank">More</a>',
						'type' => 'select',
						'default' => '=',
						'options' => array(
							esc_attr('=') => '=',
							esc_attr('!=') => '!=',
							esc_attr('>') => '>',
							esc_attr('>=') => '>=',
							esc_attr('<') => '<',
							esc_attr('<=') => '<=',
							'LIKE' => 'LIKE',						
							'NOT LIKE' => 'NOT LIKE',
							'IN' => 'IN',												
							'NOT IN' => 'NOT IN',						
							'BETWEEN' => 'BETWEEN',
							'NOT BETWEEN' => 'NOT BETWEEN',
							'EXISTS' => 'EXISTS',
							'NOT EXISTS' => 'NOT EXISTS',						
						)
					),	
					
					array(
						'id' => 'custom_field_orderby_param',
						'name' => 'Orderby parameter', 
						'desc' => 'Sort custom fields by parameter. Defaults to "date".',
						'type' => 'select',
						'default' => 'meta_value',
						'options' => array(
							'pm.meta_value' => 'Order by custom field value',
							'p.ID' => 'Order by related post id',
							'p.post_title' => 'Order by related post title',
							'p.post_name' => 'Order by related post name (post slug)',
							'p.post_date' => 'Order by related post date',
						)
					),
					array(
						'id' => 'custom_field_order_param',
						'name' => 'Order parameter', 
						'desc' => 'Designates the ascending or descending order of the "orderby" parameter. Defaults to "DESC".',
						'type' => 'radio',
						'default' => 'DESC',
						'options' => array(
							'ASC' => 'Ascending order from lowest to highest values',
							'DESC' => 'Descending order from highest to lowest values'
						)
					),
								
					array(
						'id' => 'custom_field_search_all_option',
						'name' => '"Search All" Option', 
						'desc' => 'Select "Yes" to enable this option for the "" taxonomy. Default "No"',
						'type' => 'radio',
						'default' => 'no',
						'options' => array(
							'yes' => 'Yes',
							'no' => 'No'
						)
					),
					array(
						'id' => 'custom_field_search_all_text',
						'name' => '"Search All" Label', 
						'desc' => 'Enter yout custom label for the "search all" option.',
						'type' => 'text',
						'default' => 'All',
					),
					array(
						'id' => 'custom_field_show_count',
						'name' => 'Show count', 
						'desc' => 'Select "Yes" to show the number of posts assigned to the custom field. Default "No".',
						'type' => 'radio',
						'default' => 'no',
						'options' => array(
							'yes' => 'Yes',
							'no' => 'No'
						)
					),
					array(
						'id' => 'custom_field_display_type',
						'name' => 'Display type', 
						'desc' => 'Select the display type of the custom field. Default "checkbox".',
						'type' => 'radio',
						'default' => 'checkbox',
						'options' => array(
							'checkbox' => 'Checkbox',
							'radio' => 'Radio',
							'select' => 'Select',
							'number' => 'Number <sup>Use it only for numeric values</sup>',
							'double_slider' => 'Double range slider <sup>Use it only for numeric values</sup>',
							'single_slider' => 'Single range slider <sup>Use it only for numeric values</sup>',
							'datepicker' => 'Datepicker', //@since 2.4
						)
					),

					/**
					 * Range Datepicker parameters/settings 
					 * Visible only when the "Display type" is "Datepicker"!
					 * @since 2.4 */

					array(
						'id' => 'cf_datepicker_date_format',
						'name' => 'Custom field date format', 
						'desc' => 'Specify the custom field date format. Defaults to "Year, Month, Day".',
						'type' => 'select',
						'default' => 'YYYY/MM/DD',
						'options' => array(
							'YYYY/MM/DD' => 'YYYY/MM/DD (e.g. 2017/09/12)',							
							'YY/MM/DD' => 'YY/MM/DD (e.g. 17/09/12)',
							'DD/MM/YYYY' => 'DD/MM/YYYY (e.g. 09/12/2017)',
							'DD/MM/YY' => 'DD/MM/YY (e.g. 09/12/17)',
						),
						'attributes' => array(
							'data-conditional-id' => wp_json_encode( array( $group_id, 'custom_field_display_type' ) ),
							'data-conditional-value' => 'datepicker',																									
						),													
					),
					array(
						'id' => 'cf_datepicker_date_separator',
						'name' => 'Date components separator', 
						'desc' => 'Specify the separator of the date components. Defaults to "Slash (/)".',
						'type' => 'select',
						'default' => 'slash',
						'options' => array(
							'slash' => 'Slash (/)',							
							'dash' => 'Dash (-)',
							'dot' => 'Dot (.)',
							'space' => 'Space',
							'none' => 'No separator',
						),
						'attributes' => array(
							'data-conditional-id' => wp_json_encode( array( $group_id, 'custom_field_display_type' ) ),
							'data-conditional-value' => 'datepicker',																									
						),													
					),
					array(
						'id' => 'cf_datepicker_start_date',
						'name'   => 'Start date',
						'desc' => 'Select the datepicker start view date. All the dates before this date will be disabled.',
						'type' => 'text_date',
						'date_format' => 'Y/m/d',
						'attributes' => array(
							'data-conditional-id' => wp_json_encode( array( $group_id, 'custom_field_display_type' ) ),
							'data-conditional-value' => 'datepicker',	
						),																			
					),
					array(
						'id' => 'cf_datepicker_end_date',
						'name'   => 'End date',
						'desc' => 'Select the datepicker end view date. All the dates after this date will be disabled.',
						'type' => 'text_date',
						'date_format' => 'Y/m/d',
						'attributes' => array(
							'data-conditional-id' => wp_json_encode( array( $group_id, 'custom_field_display_type' ) ),
							'data-conditional-value' => 'datepicker',
						),																			
					), 
					array(
						'id' => 'cf_datepicker_week_start',
						'name' => 'Week start',
						'desc' => 'Select the start day of the week. Defaults to "Sunday".',
						'type' => 'radio',
						'default' => 'sunday',
						'options' => array(
							'sunday' => 'Sunday',
							'monday' => 'Monday',
						),
						'attributes' => array(
							'data-conditional-id' => wp_json_encode( array( $group_id, 'custom_field_display_type' ) ),
							'data-conditional-value' => 'datepicker',																									
						),													
					),
					
					/**
					 * End range datepicker parameters */
								
					array(
						'id' => 'custom_field_display_status',
						'name' => 'Display status',
						'desc' => 'Choose whether to open this field container on page load or to close it. Defaults to "Open".',
						'type' => 'radio',
						'default' => 'open',
						'options' => array(
							'open' => 'Open',
							'close' => 'Close'
						)
					),					
					array(
						'id' => 'custom_field_visibilty',
						'name' => 'Visibilty', 
						'desc' => 'Show/hide the custom field from being displayed in the filter form. Default "Yes"',
						'type' => 'radio',
						'default' => 'yes',
						'options' => array(
							'yes' => 'Show',
							'no' => 'Hide'
						)
					)
				)
			);
			
			$fields[] = array(
				'id' => $this->metafield_prefix . '_cslf_custom_field_relation_param',
				'name' => '"Relation" parameter', 
				'desc' => 'Select the relationship between multiple custom fields. Defaults to "AND". <a href="http://codex.wordpress.org/Class_Reference/WP_Query#Custom_Field_Parameters" target="_blank">More</a>',
				'type' => 'radio',
				'default' => 'AND',
				'options' => array(
					'AND' => 'AND',
					'OR' => 'OR',
				)
			);
					
			/**
			 * Filter customiztion */
			 	
			$fields[] = array(
				'id' => $this->metafield_prefix . '_customization_section',
				'name' => 'Custmizations',
				'desc' => '',
				'type' => 'title',
				'attributes' => array(
					'style' => 'font-size:15px; color:#ff6600; font-weight:600;'
				),
			);

			$fields[] = array(
				'id' => $this->metafield_prefix . '_cslf_filter_btn_text',
				'name' => 'Filter button text', 
				'desc' => 'Enter your customized text for the "Filter" button.',
				'type' => 'text',
				'default' => 'Filter',
			);

			return $fields;
			
		}
		
		
		/**		 
		 * Get all Taxonomies related to a given post type
		 * 
		 * Since 1.0
		 */
		function cspml_get_post_type_taxonomies($post_type){
			
			$taxonomies_fields = $taxonomy_options = array();
			
			$post_type_taxonomies = (array) get_object_taxonomies($post_type, 'objects');
			
			foreach($post_type_taxonomies as $single_taxonomy){
				
				$tax_name = $single_taxonomy->name;
				$tax_label = $single_taxonomy->labels->name;	

				$taxonomy_options[$tax_name] = $tax_label;
				
			}
			
			return $taxonomy_options;
				
		}
		
		
		/**
		 * Gets a number of terms and displays them as options
		 *
		 * @since 1.0
		 */
		function cspml_get_term_options($tax_name){

			$terms = get_terms($tax_name, "hide_empty=0");
			
			$term_options = array();
						
			if(count($terms) > 0){	
				
				foreach($terms as $term){			   											
					$term_options[$term->term_id] = $term->name;
				}
				
			}
						
			return $term_options;
			
		}
				
	}
	
}	
		
